@@ -4,11 +4,28 @@
 import os
 import locale
 
-VERSION_TUPLE = (2, 2, 1)
+if os.name == "nt":
+    from win32com.shell import shellcon, shell
+
+VERSION_TUPLE = (2, 3)
 VERSION = ".".join(map(str, VERSION_TUPLE))
 
-HOME    = os.path.expanduser("~")
-USERDIR = os.path.join(HOME, ".quodlibet")
+# expanduser doesn't work with unicode on win...
+if os.name == "nt":
+    # the last 0 means SHGFP_TYPE_CURRENT
+    HOME = shell.SHGetFolderPath(0, shellcon.CSIDL_PERSONAL, 0, 0)
+else:
+    HOME = os.path.expanduser("~")
+
+if 'QUODLIBET_USERDIR' in os.environ:
+    USERDIR = os.environ['QUODLIBET_USERDIR']
+else:
+    if os.name == "nt":
+        USERDIR = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0)
+        USERDIR = os.path.join(USERDIR, "Quod Libet")
+    else:
+        USERDIR = os.path.join(HOME, ".quodlibet")
+
 CONTROL = os.path.join(USERDIR, "control")
 CONFIG  = os.path.join(USERDIR, "config")
 CURRENT = os.path.join(USERDIR, "current")
@@ -20,9 +37,6 @@ DEFAULT_RATING = 0.5
 EMAIL = os.environ.get("EMAIL", "quodlibet@lists.sacredchao.net")
 
 BASEDIR = os.path.dirname(os.path.realpath(__file__))
-if 'library.zip' in BASEDIR:
-    # Running from a py2exe distribution
-    BASEDIR = os.path.join(os.getcwd(), 'quodlibet')
 IMAGEDIR = os.path.join(BASEDIR, "images")
 
 AUTHORS = sorted("""\
@@ -66,6 +80,7 @@ Türerkan İnce
 Philipp Weis
 Johan Hovold
 Bastien Gorissen
+Nick Boultbee
 """.strip().split("\n"))
 
 ARTISTS = sorted("""\
