@@ -9,6 +9,7 @@
 # more readable, unless they're also faster.
 
 import os
+import stat
 import glob
 import shutil
 import time
@@ -17,13 +18,14 @@ from quodlibet import const
 from quodlibet import util
 
 from quodlibet.util.uri import URI
-
+from quodlibet.util import human_sort_key as human
 from quodlibet.util.tags import STANDARD_TAGS as USEFUL_TAGS
 from quodlibet.util.tags import MACHINE_TAGS
 from quodlibet.util.titlecase import title
 
-MIGRATE = ("~#playcount ~#laststarted ~#lastplayed ~#added "
-           "~#skipcount ~#rating ~bookmark").split()
+MIGRATE = frozenset(("~#playcount ~#laststarted ~#lastplayed ~#added "
+           "~#skipcount ~#rating ~bookmark").split())
+
 PEOPLE = ("albumartist artist author composer ~performers originalartist "
           "lyricist arranger conductor").split()
 
@@ -35,6 +37,9 @@ TAG_TO_SORT = {
     "~performerssort": "~performerssort"
     }
 
+INTERN_NUM_DEFAULT = frozenset("~#lastplayed ~#laststarted ~#playcount "
+    "~#skipcount ~#length ~#bitrate ~#filesize".split())
+
 SORT_TO_TAG = dict([(v, k) for (k, v) in TAG_TO_SORT.iteritems()])
 
 PEOPLE_SORT = [TAG_TO_SORT.get(k, k) for k in PEOPLE]
@@ -55,21 +60,59 @@ class AudioFile(dict):
 
     format = "Unknown Audio File"
 
-    def __sort_key(self):
-        return (self("albumsort", ""),
-                self.get("labelid") or self.get("musicbrainz_albumid"),
-                self("~#disc", self.get("discnumber")),
-                self("~#track", self.get("tracknumber")),
-                self("artistsort"), self.get("musicbrainz_artistid"),
-                self.get("title"),
-                self.get("~filename"))
-    sort_key = property(__sort_key)
+    @util.cached_property
+    def sort_key(self):
+        return (self.album_key,
+            self("~#disc"), self("~#track"),
+            human(self("artistsort")),
+            self.get("musicbrainz_artistid", ""),
+            human(self.get("title", "")),
+            self.get("~filename"))
+
+    @staticmethod
+    def sort_by_func(tag):
+        """Returns a fast sort function for a specific tag (or pattern).
+        Some keys are already in the sort cache, so we can use them."""
+        def title_sort(song):
+            key = song.sort_key
+            return (key[5], key)
+        def artist_sort(song):
+            key = song.sort_key
+            return (key[3], key)
+
+        if callable(tag):
+            return lambda song: (human(tag(song)), song.sort_key)
+        elif tag == "albumsort":
+            return lambda song: song.sort_key
+        elif tag == "title":
+            return title_sort
+        elif tag == "artistsort":
+            return artist_sort
+        elif tag.startswith("~#") and "~" not in tag[2:]:
+            return lambda song: (song(tag), song.sort_key)
+        return lambda song: (human(song(tag)), song.sort_key)
+
+    def __getstate__(self):
+        """Don't pickle anything from __dict__"""
+        pass
+
+    def __setstate__(self, state):
+        """Needed because we have defined getstate"""
+        pass
+
+    def __setitem__(self, key, value):
+        dict.__setitem__(self, key, value)
+        self.__dict__.pop("sort_key", None)
+
+    def __delitem__(self, key):
+        dict.__delitem__(self, key)
+        self.__dict__.pop("sort_key", None)
 
     key = property(lambda self: self["~filename"])
     mountpoint = property(lambda self: self["~mountpoint"])
 
     def __album_key(self):
-        return (self("albumsort", ""),
+        return (human(self("albumsort", "")),
                 self.get("album_grouping_key") or self.get("labelid") or
                 self.get("musicbrainz_albumid") or "")
     album_key = property(__album_key)
@@ -122,10 +165,12 @@ class AudioFile(dict):
         if key[:1] == "~":
             key = key[1:]
             if "~" in key:
+                if not isinstance(default, basestring): return default
                 return connector.join(
-                    map(lambda x: isinstance(x, basestring) and x or str(x),
                     filter(None,
-                    map(self.__call__, util.tagsplit("~" + key)))))
+                    map(lambda x: isinstance(x, basestring) and x or str(x),
+                    map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
+                    map(self.__call__, util.tagsplit("~" + key)))))) or default
             elif key == "#track":
                 try: return int(self["tracknumber"].split("/")[0])
                 except (ValueError, TypeError, KeyError): return default
@@ -133,13 +178,17 @@ class AudioFile(dict):
                 try: return int(self["discnumber"].split("/")[0])
                 except (ValueError, TypeError, KeyError): return default
             elif key == "length":
-                if self.get("~#length", 0) == 0: return default
-                else: return util.format_time(self.get("~#length", 0))
+                length = self.get("~#length")
+                if length is None: return default
+                else: return util.format_time(length)
+            elif key == "#rating":
+                return dict.get(self, "~" + key, const.DEFAULT_RATING)
             elif key == "rating":
-                return util.format_rating(self.get("~#rating", 0))
+                return util.format_rating(self("~#rating"))
             elif key == "people":
                 join = "\n".join
                 people = filter(None, map(self.__call__, PEOPLE))
+                if not people: return default
                 people = join(people).split("\n")
                 index = people.index
                 return join([person for (i, person) in enumerate(people)
@@ -170,11 +219,12 @@ class AudioFile(dict):
                         i = 0
                         for role in performers[performer]:
                             if i > 0:
-                                roles += ', '                            
+                                roles += ', '
                             roles += role
                             i += 1
                         values.append("%s (%s)" % (performer, roles))
                 values.extend(self.list("performer"))
+                if not values: return default
                 return "\n".join(values)
             elif key == "performerssort" or key == "performersort":
                 values = []
@@ -193,7 +243,7 @@ class AudioFile(dict):
             elif key == "uri":
                 try: return self["~uri"]
                 except KeyError:
-                    return URI.frompath(self["~filename"])
+                    return URI.frompath(util.fsdecode(self["~filename"]))
             elif key == "format":
                 return self.get("~format", self.format)
             elif key == "year":
@@ -213,9 +263,18 @@ class AudioFile(dict):
                 try: fileobj = file(self.lyric_filename, "rU")
                 except EnvironmentError: return default
                 else: return fileobj.read().decode("utf-8", "replace")
-            elif key[:1] == "#" and "~" + key not in self:
-                try: return int(self[key[1:]])
-                except (ValueError, TypeError, KeyError): return default
+            elif key[:1] == "#":
+                key = "~" + key
+                if key in self: self[key]
+                elif key in INTERN_NUM_DEFAULT:
+                    return dict.get(self, key, 0)
+                else:
+                    try: val = self[key[2:]]
+                    except KeyError: return default
+                    try: return int(val)
+                    except ValueError:
+                        try: return float(val)
+                        except ValueError: return default
             else: return dict.get(self, "~" + key, default)
 
         elif key == "title":
@@ -232,7 +291,7 @@ class AudioFile(dict):
         return dict.get(self, key, default)
 
     lyric_filename = property(lambda self: util.fsencode(
-        os.path.join(os.path.expanduser("~/.lyrics"),
+        os.path.join(util.expanduser("~/.lyrics"),
                      self.comma("artist").replace('/', '')[:128],
                      self.comma("title").replace('/', '')[:128] + '.lyric')))
 
@@ -243,8 +302,7 @@ class AudioFile(dict):
 
         if "~" in key or key == "title": v = self(key, u"")
         else: v = self.get(key, u"")
-        if isinstance(v, int): return v
-        elif isinstance(v, float): return v
+        if isinstance(v, (int, long, float)): return v
         else: return v.replace("\n", ", ")
 
     def list(self, key):
@@ -261,6 +319,23 @@ class AudioFile(dict):
         elif key in self: return self[key].split("\n")
         else: return []
 
+    def list_separate(self, key, connector=" - "):
+        """Similar to list, but will return a list of all combinations
+        for tied tags instead of one comma separated string"""
+        if key[:1] == "~" and "~" in key[1:]:
+            vals = \
+                filter(None,
+                map(lambda x: isinstance(x, basestring) and x or str(x),
+                map(lambda x: (isinstance(x, float) and "%.2f" % x) or x,
+                (self(tag) for tag in util.tagsplit(key)))))
+            vals = (val.split("\n") for val in vals)
+            r = [[]]
+            for x in vals:
+                r = [ i + [y] for y in x for i in r ]
+            return map(connector.join, r)
+        else:
+            return self.list(key)
+
     def exists(self):
         """Return true if the file still exists (or we can't tell)."""
 
@@ -331,13 +406,16 @@ class AudioFile(dict):
             return text
 
     def sanitize(self, filename=None):
-        """Fill in metadata defaults. Find ~mountpoint, ~#mtime,
+        """Fill in metadata defaults. Find ~mountpoint, ~#mtime, ~#filesize
         and ~#added. Check for null bytes in tags."""
 
         # Replace nulls with newlines, trimming zero-length segments
         for key, val in self.items():
             if isinstance(val, basestring) and '\0' in val:
                 self[key] = '\n'.join(filter(lambda s: s, val.split('\0')))
+            # Remove unnecessary defaults
+            if key in INTERN_NUM_DEFAULT and val == 0:
+                del self[key]
 
         if filename: self["~filename"] = filename
         elif "~filename" not in self: raise ValueError("Unknown filename!")
@@ -354,21 +432,21 @@ class AudioFile(dict):
         else: self["~mountpoint"] = "/"
 
         # Fill in necessary values.
-        self.setdefault("~#lastplayed", 0)
-        self.setdefault("~#laststarted", 0)
-        self.setdefault("~#playcount", 0)
-        self.setdefault("~#skipcount", 0)
-        self.setdefault("~#length", 0)
-        self.setdefault("~#bitrate", 0)
-        self.setdefault("~#rating", const.DEFAULT_RATING)
         self.setdefault("~#added", int(time.time()))
 
-        self["~#mtime"] = util.mtime(self['~filename'])
+        # For efficiency, do a single stat here. See Issue 504
+        try:
+            stat = os.stat(self['~filename'])
+            self["~#mtime"] = stat.st_mtime
+            self["~#filesize"] = stat.st_size
+        except OSError:
+            self["~#mtime"] = 0
 
     def to_dump(self):
         """A string of 'key=value' lines, similar to vorbiscomment output."""
         s = []
         for k in self.keys():
+            k = str(k)
             if isinstance(self[k], int) or isinstance(self[k], long):
                 s.append("%s=%d" % (k, self[k]))
             elif isinstance(self[k], float):
@@ -379,6 +457,10 @@ class AudioFile(dict):
                         s.append("%s=%s" % (k, v2))
                     else:
                         s.append("%s=%s" % (k, util.encode(v2)))
+        for k in (INTERN_NUM_DEFAULT - set(self.keys())):
+            s.append("%s=%d" % (k, self.get(k, 0)))
+        if "~#rating" not in self:
+            s.append("~#rating=%f" % self("~#rating"))
         s.append("~format=%s" % self.format)
         s.append("")
         return "\n".join(s)
@@ -414,52 +496,67 @@ class AudioFile(dict):
                 if key in self: del(self[key])
 
     # These should remain outside the loop below for performance reasons
-    __cover_subdirs = map(util.make_case_insensitive,
-                          ["", "scan", "scans", "images", "covers"])
-    __cover_exts = map(util.make_case_insensitive,
-                       ["jpg", "jpeg", "png", "gif"])
+    __cover_subdirs = frozenset(
+        ["scan", "scans", "images", "covers", "artwork"])
+    __cover_exts = frozenset(["jpg", "jpeg", "png", "gif"])
+
+    __cover_score = ("front", "cover", "jacket",
+        "folder", "albumart", "edited")
 
     def find_cover(self):
         """Return a file-like containing cover image data, or None if
         no cover is available."""
 
+        if not self.is_file: return
+
         base = self('~dirname')
+        get_ext = lambda s: os.path.splitext(s)[1].lstrip('.')
+
+        entries = []
+        try: entries = os.listdir(base)
+        except EnvironmentError: pass
+
         fns = []
-        # We can't pass 'base' alone to glob.glob because it probably
-        # contains confusing metacharacters, and Python's glob doesn't
-        # support any kind of escaping, so chdir and use relative
-        # paths with known safe strings.
-        try:
-            olddir = os.getcwd()
-            os.chdir(base)
-        except EnvironmentError:
-            pass
-        else:
-            for subdir in self.__cover_subdirs:
-                for ext in self.__cover_exts:
-                    fns.extend(glob.glob(os.path.join(subdir, u"*." + ext)))
-                    fns.extend(glob.glob(os.path.join(subdir, u".*." + ext)))
-            os.chdir(olddir)
+        for entry in entries:
+            lentry = entry.lower()
+            if get_ext(lentry) in self.__cover_exts:
+                fns.append((None, entry))
+            if lentry in self.__cover_subdirs:
+                subdir = os.path.join(base, entry)
+                sub_entries = []
+                try: sub_entries = os.listdir(subdir)
+                except EnvironmentError: pass
+                for sub_entry in sub_entries:
+                    lsub_entry = sub_entry.lower()
+                    if get_ext(lsub_entry) in self.__cover_exts:
+                        fns.append((entry, sub_entry))
+
         images = []
-        for fn in sorted(fns):
-            lfn = fn.lower()
+        for sub, fn in fns:
             score = 0
+            lfn = fn.lower()
             # check for the album label number
-            if self.get("labelid", fn + ".").lower() in lfn: score += 100
-            score += sum(map(lfn.__contains__,
-                    ["front", "cover", "jacket", "folder", "albumart"]))
+            if "labelid" in self and self["labelid"].lower() in lfn:
+                score += 100
+            score += sum(map(lfn.__contains__, self.__cover_score))
             if score:
+                if sub is not None:
+                    fn = os.path.join(sub, fn)
                 images.append((score, os.path.join(base, fn)))
-        # Highest score wins.
-        if images:
-            try:
-                return file(util.fsencode(max(images)[1]), "rb")
-            except IOError:
-                return None
-        elif "~picture" in self:
+        images.sort(reverse=True)
+
+        for score, path in images:
+            # could be a directory
+            if not os.path.isfile(path):
+                continue
+            try: return file(path, "rb")
+            except IOError: pass
+
+        if "~picture" in self:
             # Otherwise, we might have a picture stored in the metadata...
             return self.get_format_cover()
-        else: return None
+
+        return None
 
     def replay_gain(self, profiles, pre_amp_gain=0, fallback_gain=0):
         """Return the computed Replay Gain scale factor.
@@ -476,7 +573,7 @@ class AudioFile(dict):
             try:
                 db = float(self["replaygain_%s_gain" % profile].split()[0])
                 peak = float(self.get("replaygain_%s_peak" % profile, 1))
-            except (KeyError, ValueError):
+            except (KeyError, ValueError, IndexError):
                 continue
             else:
                 db += pre_amp_gain
