@@ -23,7 +23,7 @@ class NullPlayer(BasePlayer):
         """Return the current playback position in milliseconds,
         or 0 if no song is playing."""
         return 0
-        
+
     def _set_paused(self, paused):
         if paused != self._paused:
             self._paused = paused
@@ -65,7 +65,7 @@ class NullPlayer(BasePlayer):
         self.emit('song-started', self.song)
 
         if self.song is not None:
-            self._length = self.song["~#length"] * 1000
+            self._length = self.song.get("~#length", 0) * 1000
         else:
             self.paused = True
 
