@@ -4,8 +4,6 @@
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
-#
-# $Id: about.py 7fea50f0f7c9 2007/06/18 07:00:29 $
 
 import gtk
 import mutagen
@@ -17,6 +15,7 @@ from quodlibet.util import fver
 class AboutQuodLibet(gtk.AboutDialog):
     def __init__(self, parent, player):
         super(AboutQuodLibet, self).__init__()
+        self.set_transient_for(parent)
         self.set_name("Quod Libet")
         self.set_version(const.VERSION)
         self.set_authors(const.AUTHORS)
@@ -33,9 +32,9 @@ class AboutQuodLibet(gtk.AboutDialog):
         # Translators: Replace this with your name/email to have it appear
         # in the "About" dialog.
         self.set_translator_credits(_('translator-credits'))
-        self.set_website("http://www.sacredchao.net/quodlibet")
+        self.set_website("http://code.google.com/p/quodlibet")
         self.set_copyright(
-            "Copyright © 2004-2007 Joe Wreschnig, Michael Urman, & others\n"
+            "Copyright © 2004-2009 Joe Wreschnig, Michael Urman, & others\n"
             "<quodlibet@lists.sacredchao.net>")
         self.child.show_all()
 
