@@ -12,7 +12,7 @@ import pango
 from quodlibet import qltk
 
 from quodlibet.qltk.views import RCMHintedTreeView
-from quodlibet.qltk import ClearButton
+from quodlibet.qltk.entry import ValidatingNoSexyEntry
 
 class CBESEditor(qltk.Window):
     def __init__(self, cbes):
@@ -155,15 +155,18 @@ class ComboBoxEntrySave(gtk.ComboBoxEntry):
     """A ComboBoxEntry that remembers the past 'count' strings entered,
     and can save itself to (and load itself from) a filename or file-like."""
 
-    models = {}
+    __models = {}
+    __last = ""
 
-    def __init__(self, filename=None, initial=[], count=5, model=None):
+    def __init__(self, filename=None, initial=[], count=5, id=None,
+        validator=None):
         self.count = count
         self.filename = filename
-        try: model = self.models[model]
+        id = filename or id
+
+        try: model = self.__models[id]
         except KeyError:
-            model = self.models[model] = gtk.ListStore(str, str, str)
-        else: model = gtk.ListStore(str, str, str)
+            model = type(self).__models[id] = gtk.ListStore(str, str, str)
 
         super(ComboBoxEntrySave, self).__init__(model, 0)
         self.clear()
@@ -178,30 +181,29 @@ class ComboBoxEntrySave(gtk.ComboBoxEntry):
 
         self.set_row_separator_func(self.__separator_func)
 
-        if len(model) == 0:
+        if not len(model):
             self.__fill(filename, initial)
+
+        self.remove(self.child)
+        self.add(ValidatingNoSexyEntry(validator))
+
         self.connect_object('destroy', self.set_model, None)
         self.connect_object('changed', self.__changed, model)
 
-    def pack_clear_button(self, container=None):
-        if getattr(self.child, "set_icon_from_stock", None):
-            self.child.set_icon_from_stock(
-                gtk.ENTRY_ICON_SECONDARY, gtk.STOCK_CLEAR)
-            clear = lambda *x: x[0].set_text("")
-            self.child.connect("icon-release", clear)
-        else:
-            container.pack_start(ClearButton(self.child), False)
+    def pack_clear_button(self, *args):
+        self.child.pack_clear_button(*args)
 
     def __changed(self, model):
         iter = self.get_active_iter()
         if iter:
             if model[iter][2] in ICONS:
+                self.child.set_text(self.__last)
                 Kind = ICONS[model[iter][2]]
                 Kind(self)
                 self.set_active(-1)
             else:
                 self.__focus_entry()
-        model[self.__special_iter][0] = self.child.get_text()
+        self.__last = self.child.get_text()
 
     def __focus_entry(self):
         self.child.grab_focus()
@@ -209,8 +211,7 @@ class ComboBoxEntrySave(gtk.ComboBoxEntry):
 
     def __fill(self, filename, initial):
         model = self.get_model()
-        self.__special_iter = model.append(
-            row=["", _("Edit saved values..."), gtk.STOCK_EDIT])
+        model.append(row=["", _("Edit saved values..."), gtk.STOCK_EDIT])
         model.append(row=[None, None, None])
 
         if filename is None: return
