@@ -35,6 +35,7 @@ from quodlibet.qltk.information import Information
 from quodlibet.qltk.logging import LoggingWindow
 from quodlibet.qltk.mmkeys_ import MmKeys
 from quodlibet.qltk.msg import ErrorMessage
+from quodlibet.qltk.notif import StatusBar, TaskController
 from quodlibet.qltk.playorder import PlayOrder
 from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.properties import SongProperties
@@ -44,107 +45,82 @@ from quodlibet.qltk.songlist import SongList, PlaylistMux
 from quodlibet.qltk.x import RPaned
 from quodlibet.util import copool
 from quodlibet.util.uri import URI
+from quodlibet.util.library import background_filter
 
 class MainSongList(SongList):
     # The SongList that represents the current playlist.
 
     class CurrentColumn(gtk.TreeViewColumn):
         # Displays the current song indicator, either a play or pause icon.
-    
-        _render = gtk.CellRendererPixbuf()
-        _render.set_property('xalign', 0.5)
         header_name = "~current"
+        __last_stock = None
 
         def _cdf(self, column, cell, model, iter,
                  pixbuf=(gtk.STOCK_MEDIA_PLAY, gtk.STOCK_MEDIA_PAUSE)):
-            try:
-                if model.get_path(iter) == model.current_path:
-                    if model.sourced:
-                        stock = pixbuf[player.playlist.paused]
-                    else:
-                        stock = gtk.STOCK_MEDIA_STOP
-                elif model[iter][0].get("~errors"):
-                    stock = gtk.STOCK_DIALOG_ERROR
-                elif model.get_path(iter) != model.current_path:
-                    stock = ''
-                cell.set_property('stock-id', stock)
-            except AttributeError: pass
+            row = model[iter]
+            if row.path == model.current_path:
+                if model.sourced:
+                    stock = pixbuf[player.playlist.paused]
+                else:
+                    stock = gtk.STOCK_MEDIA_STOP
+            elif row[0].get("~errors"):
+                stock = gtk.STOCK_DIALOG_ERROR
+            else:
+                stock = ''
+            if self.__last_stock == stock: return
+            self.__last_stock = stock
+            cell.set_property('stock-id', stock)
 
         def __init__(self):
+            self._render = gtk.CellRendererPixbuf()
+            self._render.set_property('xalign', 0.5)
             super(MainSongList.CurrentColumn, self).__init__("", self._render)
             self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             self.set_fixed_width(24)
             self.set_cell_data_func(self._render, self._cdf)
             self.header_name = "~current"
 
-    def __init__(self, library, player, visible):
-        super(MainSongList, self).__init__(library, player)
-        self.set_rules_hint(True)
-        s = library.librarian.connect_object('removed', map, player.remove)
-        self.connect_object('destroy', library.librarian.disconnect, s)
-        self.connect_object('row-activated', self.__select_song, player)
-        self.connect_object('notify::visible', self.__visibility, visible)
+    _activated = False
 
-    def __visibility(self, visible, event):
-        visible.set_active(self.get_property('visible'))
+    def __init__(self, library, player):
+        super(MainSongList, self).__init__(library, player)
+
+        self.connect_object('row-activated', self.__select_song, player)
+
+        # ugly.. so the main window knows if the next song-started
+        # comes from an row-activated or anything else.
+        def reset_activated(*args):
+            self._activated = False
+        s = player.connect_after('song-started', reset_activated)
+        self.connect_object('destroy', player.disconnect, s)
 
     def __select_song(self, player, indices, col):
+        self._activated = True
         iter = self.model.get_iter(indices)
-        player.go_to(iter)
-        if player.song: player.paused = False
+        if player.go_to(iter, True):
+            player.paused = False
 
     def set_sort_by(self, *args, **kwargs):
         super(MainSongList, self).set_sort_by(*args, **kwargs)
         tag, reverse = self.get_sort_by()
         config.set('memory', 'sortby', "%d%s" % (int(reverse), tag))
 
-class StatusBar(gtk.HBox):
-    def __init__(self):
-        super(StatusBar, self).__init__()
-        self.progress = gtk.ProgressBar()
-        self.count = gtk.Label(_("No time information"))
-        self.count.set_alignment(1.0, 0.5)
-        self.count.set_ellipsize(pango.ELLIPSIZE_START)
-        self.pack_start(self.count)
-        progress_label = gtk.Label()
-        progress_label.set_alignment(1.0, 0.5)
-        progress_label.set_ellipsize(pango.ELLIPSIZE_START)
-        # GtkProgressBar can't show text when pulsing. Proxy its set_text
-        # method to a label that can.
-        self.progress.set_text = progress_label.set_text
-        hb = gtk.HBox(spacing=12)
-        hb.pack_start(progress_label)
-        hb.pack_start(self.progress, expand=False)
-        pause = gtk.ToggleButton()
-        pause.add(gtk.image_new_from_stock(
-            gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_MENU))
-        pause.connect('toggled', self.__pause)
-        hb.pack_start(pause, expand=False)
-        self.pack_start(hb)
-        self.progress.connect('notify::visible', self.__toggle, pause, hb)
-        self.progress.connect_object(
-            'notify::fraction', lambda *args: args[0].set_active(False), pause)
-        self.count.show()
+class SongListScroller(gtk.ScrolledWindow):
+    def __init__(self, menu):
+        super(SongListScroller, self).__init__()
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+        self.set_shadow_type(gtk.SHADOW_IN)
+        self.connect_object('notify::visible', self.__visibility, menu)
 
-    def __pause(self, pause):
-        if pause.get_active():
-            copool.pause("library")
-        else:
-            copool.resume("library")
-
-    def __toggle(self, bar, property, pause, hb):
-        if self.progress.props.visible:
-            self.count.hide()
-            pause.set_active(False)
-            hb.show_all()
-        else:
-            self.count.show()
-            hb.hide()
+    def __visibility(self, menu, event):
+        value = self.get_property('visible')
+        menu.set_active(value)
+        config.set("memory", "songlist", str(value))
 
 class QuodLibetWindow(gtk.Window):
     def __init__(self, library, player):
         super(QuodLibetWindow, self).__init__()
-        self.last_dir = os.path.expanduser("~")
+        self.last_dir = const.HOME
 
         self.set_title("Quod Libet")
 
@@ -172,11 +148,14 @@ class QuodLibetWindow(gtk.Window):
         self.child.pack_start(realvbox)
 
         # get the playlist up before other stuff
-        self.songlist = MainSongList(
-            library, player, self.ui.get_widget("/Menu/View/SongList"))
+        self.songlist = MainSongList(library, player)
+        self.songlist.show_all()
         self.add_accel_group(self.songlist.accelerators)
         self.songlist.connect_after(
             'drag-data-received', self.__songlist_drag_data_recv)
+        self.song_scroller = SongListScroller(
+            self.ui.get_widget("/Menu/View/SongList"))
+        self.song_scroller.add(self.songlist)
         self.qexpander = QueueExpander(
             self.ui.get_widget("/Menu/View/Queue"), library, player)
         self.playlist = PlaylistMux(
@@ -195,7 +174,7 @@ class QuodLibetWindow(gtk.Window):
         hbox.pack_start(text)
 
         # cover image
-        self.image = CoverImage()
+        self.image = CoverImage(resize=True)
         player.connect('song-started', self.image.set_song)
         hbox.pack_start(self.image, expand=False)
 
@@ -207,7 +186,7 @@ class QuodLibetWindow(gtk.Window):
         hbox = gtk.HBox(spacing=12)
         hb = gtk.HBox(spacing=3)
         label = gtk.Label(_("_Order:"))
-        label.set_size_request(-1, 28)
+        label.set_size_request(-1, 24)
         self.order = order = PlayOrder(self.songlist.model, player)
         label.set_mnemonic_widget(order)
         label.set_use_underline(True)
@@ -222,32 +201,27 @@ class QuodLibetWindow(gtk.Window):
         align.add(hbox)
         self.child.pack_end(align, expand=False)
 
-        # song list
-        self.song_scroller = sw = gtk.ScrolledWindow()
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
-        sw.set_shadow_type(gtk.SHADOW_IN)
-        sw.add(self.songlist)
+        self.songpane = gtk.VPaned()
+        self.songpane.pack1(self.song_scroller, resize=True, shrink=False)
+        self.songpane.pack2(self.qexpander, resize=True, shrink=False)
+        self.__handle_position =  self.songpane.get_property("position")
 
-        self.songpane = gtk.VBox(spacing=6)
-        self.songpane.pack_start(self.song_scroller)
-        self.songpane.pack_start(self.qexpander, expand=False, fill=True)
-        self.songpane.show_all()
         self.song_scroller.connect('notify::visible', self.__show_or)
         self.qexpander.connect('notify::visible', self.__show_or)
+        self.qexpander.connect('notify::expanded', self.__expand_or)
+        self.qexpander.connect('size-allocate', self.__qex_size_allocate)
+        self.songpane.connect('notify', self.__moved_pane_handle)
 
         sort = config.get('memory', 'sortby')
         self.songlist.set_sort_by(None, sort[1:], order=int(sort[0]))
 
-        self.inter = gtk.VBox()
-
         self.browser = None
 
         self.__keys = MmKeys(player)
 
         self.child.show_all()
-        sw.show_all()
 
-        self.statusbar = StatusBar()
+        self.statusbar = StatusBar(TaskController.default_instance)
         hbox.pack_start(self.statusbar)
         self.statusbar.show()
 
@@ -266,9 +240,11 @@ class QuodLibetWindow(gtk.Window):
         repeat.connect('toggled', self.__repeat, self.songlist.model)
         repeat.set_active(config.getboolean('settings', 'repeat'))
 
+        # track window position/size
+        self.connect("show", self.__restore_window_position)
         self.connect('configure-event', QuodLibetWindow.__save_size)
         self.connect('window-state-event', self.__window_state_changed)
-        self.__hidden_state = 0
+        self.__state = 0
 
         self.songlist.connect('popup-menu', self.__songs_popup_menu)
         self.songlist.connect('columns-changed', self.__cols_changed)
@@ -292,6 +268,8 @@ class QuodLibetWindow(gtk.Window):
         self.connect_object(
             'drag-data-received', QuodLibetWindow.__drag_data_received, self)
 
+        if config.getint("memory", "maximized"):
+            self.maximize()
         self.resize(*map(int, config.get("memory", "size").split()))
 
         if config.getboolean('library', 'refresh_on_start'):
@@ -344,47 +322,54 @@ class QuodLibetWindow(gtk.Window):
             if dirs:
                 copool.add(
                     self.__library.scan, dirs, self.__status.bar.progress,
-                    funcid="library")
+                    cofuncid="library", funcid="library")
 
     def __songlist_drag_data_recv(self, view, *args):
         if callable(self.browser.reordered): self.browser.reordered(view)
         self.songlist.set_sort_by(None, refresh=False)
 
-    def __window_state_changed(self, window, event):
-        assert window is self
-        self.__window_state = event.new_window_state
-
     def hide(self):
-        self.__hidden_state = self.__window_state
-        if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
-            self.unmaximize()
         super(QuodLibetWindow, self).hide()
         map(gtk.Window.hide, qltk.Window.childs)
 
     def present(self):
         super(QuodLibetWindow, self).present()
-        if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
-            self.maximize()
         map(gtk.Window.present, qltk.Window.childs)
 
     def show(self):
+        self.move(*map(int, config.get('memory', 'position').split()))
         super(QuodLibetWindow, self).show()
-        if self.__hidden_state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
-            self.maximize()
         map(gtk.Window.show, qltk.Window.childs)
 
     def destroy(self, *args):
+        # The tray icon plugin tries to unhide QL because it gets disabled
+        # on Ql exit. The window should be hidden after destroy gets called.
+        self.show = lambda: None
+        self.present = self.show
         super(QuodLibetWindow, self).destroy()
 
     def __show_or(self, widget, prop):
         ssv = self.song_scroller.get_property('visible')
         qxv = self.qexpander.get_property('visible')
         self.songpane.set_property('visible', ssv or qxv)
-        self.songpane.set_child_packing(
-            self.qexpander, expand=not ssv, fill=True, padding=0,
-            pack_type=gtk.PACK_START)
         if not ssv:
             self.qexpander.set_expanded(True)
+        self.__expand_or(widget, prop)
+
+    def __expand_or(self, widget, prop):
+        if self.qexpander.get_property('expanded'):
+            self.songpane.set_property("position", self.__handle_position)
+
+    def __moved_pane_handle(self, widget, prop):
+        if self.qexpander.get_property('expanded'):
+            self.__handle_position = self.songpane.get_property("position")
+
+    def __qex_size_allocate(self, event, param=None):
+        if not self.qexpander.get_property('expanded'):
+            p_max = self.songpane.get_property("max-position")
+            p_cur = self.songpane.get_property("position")
+            if p_max != p_cur:
+                self.songpane.set_property("position", p_max)
 
     def __create_menu(self, player, library):
         ag = gtk.ActionGroup('QuodLibetWindowActions')
@@ -499,7 +484,7 @@ class QuodLibetWindow(gtk.Window):
         current = browsers.index(config.get("memory", "browser"))
         ag.add_radio_actions(
             view_actions, current, self.select_browser,
-            (library, player, False))
+            (library, player))
 
         for Kind in browsers.browsers:
             if not Kind.in_menu: continue
@@ -539,7 +524,7 @@ class QuodLibetWindow(gtk.Window):
             key = "%s_pos" % browser.__class__.__name__
             config.set("browsers", key, str(paned.get_relative()))
 
-    def select_browser(self, activator, current, library, player, restore):
+    def select_browser(self, activator, current, library, player, restore=False):
         if isinstance(current, gtk.RadioAction):
             current = current.get_current_value()
         Browser = browsers.get(current)
@@ -571,6 +556,8 @@ class QuodLibetWindow(gtk.Window):
             try:
                 key = "%s_pos" % self.browser.__class__.__name__
                 val = config.getfloat("browsers", key)
+                # Use a minimum restore size
+                val = max(val, 0.1)
             except: val = 0.4
             container.connect(
                 'notify::position', self.__browser_configure, self.browser)
@@ -616,8 +603,7 @@ class QuodLibetWindow(gtk.Window):
         else: self.set_title("Quod Libet")
 
     def __song_started(self, player, song):
-        if song is None:
-            self.__update_title(player, [song])
+        self.__update_title(player, [song])
 
         for wid in ["Jump", "Next", "EditTags", "Information"]:
             self.ui.get_widget('/Menu/Control/'+wid).set_sensitive(bool(song))
@@ -628,15 +614,35 @@ class QuodLibetWindow(gtk.Window):
                 self.ui.get_widget(
                     "/Menu/Filters/Filter%s" % h.capitalize()).set_sensitive(
                     h in song)
-        if song and config.getboolean("settings", "jump"):
+
+        if song and not self.songlist._activated and \
+            config.getboolean("settings", "jump"):
             self.__jump_to_current(False)
 
+    def __window_state_changed(self, window, event):
+        self.__state = event.new_window_state
+        if self.__state & gtk.gdk.WINDOW_STATE_WITHDRAWN: return
+        if self.__state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
+            config.set("memory", "maximized", "1")
+        else:
+            config.set("memory", "maximized", "0")
+
     def __save_size(self, event):
-        config.set("memory", "size", "%d %d" % (event.width, event.height))
+        if not self.__state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
+            config.set("memory", "size", "%d %d" % (event.width, event.height))
+            if self.get_property("visible"):
+                config.set('memory', 'position', '%s %s' % self.get_position())
+
+    def __restore_window_position(self, window):
+        pos = map(int, config.get('memory', 'position').split())
+        self.move(*pos)
 
     def __refresh_size(self):
+        self.__vbox.set_spacing(6)
         if (not self.browser.expand and
-            not self.songpane.get_property('visible')):
+            not self.song_scroller.get_property('visible')):
+            if self.browser.size_request()[1] == 0:
+                self.__vbox.set_spacing(0)
             width, height = self.get_size()
             height = self.size_request()[1]
             self.resize(width, height)
@@ -646,7 +652,6 @@ class QuodLibetWindow(gtk.Window):
 
     def showhide_playlist(self, toggle):
         self.song_scroller.set_property('visible', toggle.get_active())
-        config.set("memory", "songlist", str(toggle.get_active()))
         self.__refresh_size()
 
     def showhide_playqueue(self, toggle):
@@ -659,27 +664,45 @@ class QuodLibetWindow(gtk.Window):
         else: player.playlist.paused ^= True
 
     def __jump_to_current(self, explicit):
-        if player.playlist.song is None: return
+        """Select/scroll to the current playing song in the playlist.
+        If it can't be found tell the browser to properly fill the playlist
+        with an appropriate selection containing the song.
 
-        if player.playlist.song != self.songlist.model.current and explicit:
-            self.browser.scroll(player.playlist.song)
+        explicit means that the jump request comes from the user and not
+        from an event like song-started.
+        """
 
-        #we need to wait until the browser has finished scrolling/filling
-        #and the songlist is ready
-        gobject.idle_add(self.__jump_to_current_in_list, explicit)
+        def jump_to(song, select=True):
+            model =  self.songlist.model
+            if song == model.current:
+                path = model.current_path
+            else:
+                iter = model.find(song)
+                if iter is None: return
+                path = model[iter].path
 
-    def __jump_to_current_in_list(self, explicit):
-        if player.playlist.song == self.songlist.model.current:
-            path = self.songlist.model.current_path
-            if path is None: return
-            self.songlist.scroll_to_cell(
-                path[0], use_align=True, row_align=0.5)
-            if explicit:
-                iter = self.songlist.model.current_iter
+            self.songlist.scroll_to_cell(path, use_align=True, row_align=0.5)
+            if select:
                 selection = self.songlist.get_selection()
                 selection.unselect_all()
                 selection.select_path(path)
 
+        song = player.playlist.song
+        model =  self.songlist.model
+
+        # We are not playing a song
+        if song is None: return
+
+        # model.find because the source could be the queue
+        if song == model.current or (model.find(song) and explicit):
+            jump_to(song, select=explicit)
+        elif explicit:
+            self.browser.scroll(player.playlist.song)
+            # We need to wait until the browser has finished
+            # scrolling/filling and the songlist is ready.
+            # Not perfect, but works for now.
+            gobject.idle_add(jump_to, song, priority=gobject.PRIORITY_LOW)
+
     def __next_song(self, *args): player.playlist.next()
     def __previous_song(self, *args): player.playlist.previous()
 
@@ -703,7 +726,9 @@ class QuodLibetWindow(gtk.Window):
         self.__make_query("#(playcount = 0)")
 
     def __top40(self, menuitem):
-        songs = [song["~#playcount"] for song in self.__library]
+        bg = background_filter()
+        songs = (bg and filter(bg, self.__library)) or self.__library
+        songs = [song.get("~#playcount", 0) for song in songs]
         if len(songs) == 0: return
         songs.sort()
         if len(songs) < 40:
@@ -712,11 +737,13 @@ class QuodLibetWindow(gtk.Window):
             self.__make_query("#(playcount > %d)" % (songs[-40] - 1))
 
     def __bottom40(self, menuitem):
-        songs = [song["~#playcount"] for song in self.__library]
+        bg = background_filter()
+        songs = (bg and filter(bg, self.__library)) or self.__library
+        songs = [song.get("~#playcount", 0) for song in songs]
         if len(songs) == 0: return
         songs.sort()
         if len(songs) < 40:
-            self.__make_query("#(playcount < %d)" % (songs[0] + 1))
+            self.__make_query("#(playcount < %d)" % (songs[-1] + 1))
         else:
             self.__make_query("#(playcount < %d)" % (songs[-40] + 1))
 
@@ -724,8 +751,7 @@ class QuodLibetWindow(gtk.Window):
         paths = util.split_scan_dirs(config.get("settings", "scan"))
         exclude = config.get("library", "exclude").split(":")
         copool.add(self.__library.rebuild,
-                   paths, self.statusbar.progress, force,
-                   exclude, funcid="library")
+                   paths, force, exclude, cofuncid="library", funcid="library")
 
     # Set up the preferences window.
     def __preferences(self, activator):
@@ -773,8 +799,7 @@ class QuodLibetWindow(gtk.Window):
         if fns:
             if action.get_name() == "AddFolders":
                 self.last_dir = fns[0]
-                copool.add(self.__library.scan, fns, self.statusbar.progress,
-                           funcid="library")
+                copool.add(self.__library.scan, fns, funcid="library")
             else:
                 added = []
                 self.last_dir = os.path.basename(fns[0])
@@ -840,13 +865,8 @@ class QuodLibetWindow(gtk.Window):
 
     def __browser_cb(self, browser, songs, sorted):
         if browser.background:
-            try: bg = config.get("browsers", "background").decode('utf-8')
-            except UnicodeError: bg = ""
-            if bg:
-                try: search = Query(bg, SongList.star).search
-                except Query.error: pass
-                else: songs = filter(search, songs)
-
+            bg = background_filter()
+            if bg: songs = filter(bg, songs)
         self.__set_time(songs=songs)
         self.songlist.set_songs(songs, sorted)
 
@@ -894,7 +914,7 @@ class QuodLibetWindow(gtk.Window):
         if "songs" not in kwargs and len(songs) <= 1:
             songs = self.songlist.get_songs()
         i = len(songs)
-        length = sum([song["~#length"] for song in songs])
+        length = sum([song("~#length", 0) for song in songs])
         t = self.browser.statusbar(i) % {
             'count': i, 'time': util.format_time_long(length)}
-        self.statusbar.count.set_text(t)
+        self.statusbar.set_default_text(t)
