@@ -23,7 +23,7 @@ from quodlibet.qltk.properties import SongProperties
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.util.uri import URI
 from quodlibet.qltk.playorder import ORDERS
-from quodlibet.formats._audio import TAG_TO_SORT
+from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
 
 class PlaylistMux(object):
 
@@ -67,10 +67,10 @@ class PlaylistMux(object):
     def previous(self):
         self.pl.previous()
 
-    def go_to(self, song):
+    def go_to(self, song, explicit=False):
         print_d("Told to go to %r" % song, context=self)
-        self.pl.go_to(song)
         self.q.go_to(None)
+        return self.pl.go_to(song, explicit)
 
     def reset(self):
         self.pl.reset()
@@ -100,6 +100,13 @@ class PlaylistModel(gtk.ListStore):
         super(PlaylistModel, self).__init__(object)
         self.order = ORDERS[0](self)
 
+        # The playorder plugins use paths atm to remember songs so
+        # we need to reset them if the paths change somehow.
+        self.__sigs = []
+        for sig in ['row-deleted', 'row-inserted', 'rows-reordered']:
+            s = self.connect(sig, lambda pl, *x: self.order.reset(pl))
+            self.__sigs.append(s)
+
     def set(self, songs):
         oldsong = self.current
         if oldsong is None: oldsong = self.__old_value
@@ -107,21 +114,32 @@ class PlaylistModel(gtk.ListStore):
         self.order.reset(self)
         self.__iter = None
 
+        # We just reset the order manually so block the signals
+        map(self.handler_block, self.__sigs)
         print_d("Clearing model.", context=self)
         self.clear()
         print_d("Setting %d songs." % len(songs), context=self)
-        for song in songs:
-            iter = self.append(row=[song])
-            if song == oldsong:
+        insert = self.insert
+        for song in reversed(songs):
+            iter = insert(0, (song,))
+            if song is oldsong:
                 self.__iter = iter
         if self.__iter is not None:
             self.__old_value = None
         print_d("Done filling model.", context=self)
+        map(self.handler_unblock, self.__sigs)
         self.emit('songs-set')
 
+    def reverse(self):
+        if not len(self): return
+        self.order.reset(self)
+        map(self.handler_block, self.__sigs)
+        self.reorder(range(len(self)-1, -1, -1))
+        map(self.handler_unblock, self.__sigs)
+
     def remove(self, iter):
         if self.__iter and self[iter].path == self[self.__iter].path:
-            self.__iter = self.iter_next(iter)
+            self.__iter = None
         super(PlaylistModel, self).remove(iter)
 
     def get(self):
@@ -155,7 +173,7 @@ class PlaylistModel(gtk.ListStore):
     def previous(self):
         self.__iter = self.order.previous_explicit(self, self.__iter)
 
-    def go_to(self, song):
+    def go_to(self, song, explicit=False):
         print_d("Told to go to %r" % song, context=self)
         self.__iter = None
         if isinstance(song, gtk.TreeIter):
@@ -170,7 +188,11 @@ class PlaylistModel(gtk.ListStore):
                     break
             else:
                 print_d("Failed to find song", context=self)
-        self.__iter = self.order.set_explicit(self, self.__iter)
+        if explicit:
+            self.__iter = self.order.set_explicit(self, self.__iter)
+        else:
+            self.__iter = self.order.set_implicit(self, self.__iter)
+        return self.__iter
 
     def find(self, song):
         for row in self:
@@ -184,7 +206,7 @@ class PlaylistModel(gtk.ListStore):
         return bool(self.find(song))
 
     def is_empty(self):
-        return not bool(len(self))
+        return not len(self)
 
     def reset(self):
         self.go_to(None)
@@ -200,64 +222,87 @@ class SongList(AllTreeView, util.InstanceTracker):
 
     class TextColumn(qltk.views.TreeViewColumnButton):
         # Base class for other kinds of columns.
-        _render = gtk.CellRendererText()
         _label = gtk.Label().create_pango_layout("")
-        def _cdf(self, column, cell, model, iter, tag):
-            try:
-                song = model.get_value(iter, 0)
-                text = unicode(song.comma(tag))
-                cell.set_property('text', text)
-                self._update_layout(text, cell)
-            except AttributeError: pass
+        __last_rendered = None
 
-        def _update_layout(self, text, cell=None, pad=12):
-            if not self.get_resizable():
+        def _needs_update(self, value):
+            if self.__last_rendered == value:
+                return False
+            self.__last_rendered = value
+            return True
+
+        def _cdf(self, column, cell, model, iter, tag):
+            text = model[iter][0].comma(tag)
+            if not self._needs_update(text): return
+            cell.set_property('text', text)
+            self._update_layout(text, cell)
+
+        def _delayed_update(self):
+            max_width = -1
+            width = self.get_fixed_width()
+            for text, pad, cell_pad in self._text:
                 self._label.set_text(text)
-                width = self.get_fixed_width()
-                new_width = self._label.get_pixel_size()[0] + pad
-                if cell:
-                    new_width += cell.get_property('xpad')
-                if width < new_width:
-                    self.set_fixed_width(new_width)
+                new_width = self._label.get_pixel_size()[0] + pad +  cell_pad
+                if new_width > max_width:
+                    max_width = new_width
+            if width < max_width:
+                self.set_fixed_width(max_width)
+                tv = self.get_tree_view()
+                if tv: tv.columns_autosize()
+            self._text.clear()
+            self._timeout = None
+            return False
+
+        def _update_layout(self, text, cell=None, pad=12, force=False):
+            if not self.get_resizable():
+                cell_pad = (cell and cell.get_property('xpad')) or 0
+                self._text.add((text, pad, cell_pad))
+                if force: self._delayed_update()
+                if self._timeout is not None:
+                    gobject.source_remove(self._timeout)
+                    self._timeout = None
+                self._timeout = gobject.idle_add(self._delayed_update,
+                    priority=gobject.PRIORITY_LOW)
 
         def __init__(self, t):
-            super(SongList.TextColumn, self).__init__(util.tag(t), self._render)
+            self._render = gtk.CellRendererText()
+            title = util.tag(t)
+            super(SongList.TextColumn, self).__init__(title, self._render)
             self.header_name = t
             self.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
             self.set_visible(True)
             self.set_clickable(True)
             self.set_sort_indicator(False)
             self.set_cell_data_func(self._render, self._cdf, t)
-            self._update_layout(util.tag(t))
+            self._text = set()
+            self._timeout = None
+            self._update_layout(title, force=True)
 
     class DateColumn(TextColumn):
         # The '~#' keys that are dates.
         def _cdf(self, column, cell, model, iter, tag):
-            try:
-                stamp = model.get_value(iter, 0)(tag)
-                if not stamp:
-                    cell.set_property('text', _("Never"))
-                else:
-                    date = datetime.datetime.fromtimestamp(stamp).date()
-                    today = datetime.datetime.now().date()
-                    days = (today - date).days
-                    if days == 0: format = "%X"
-                    elif days < 7: format = "%A"
-                    else: format = "%x"
-                    stamp = time.localtime(stamp)
-                    text = time.strftime(format, stamp).decode(const.ENCODING)
-                    cell.set_property('text', text)
-                self._update_layout(cell.get_property('text'), cell)
-            except AttributeError: pass
+            stamp = model[iter][0](tag)
+            if not self._needs_update(stamp): return
+            if not stamp:
+                cell.set_property('text', _("Never"))
+            else:
+                date = datetime.datetime.fromtimestamp(stamp).date()
+                today = datetime.datetime.now().date()
+                days = (today - date).days
+                if days == 0: format = "%X"
+                elif days < 7: format = "%A"
+                else: format = "%x"
+                stamp = time.localtime(stamp)
+                text = time.strftime(format, stamp).decode(const.ENCODING)
+                cell.set_property('text', text)
+            self._update_layout(cell.get_property('text'), cell)
 
     class WideTextColumn(TextColumn):
         # Resizable and ellipsized at the end. Used for any key with
         # a '~' in it, and 'title'.
-        _render = gtk.CellRendererText()
-        _render.set_property('ellipsize', pango.ELLIPSIZE_END)
-
         def __init__(self, tag):
             super(SongList.WideTextColumn, self).__init__(tag)
+            self._render.set_property('ellipsize', pango.ELLIPSIZE_END)
             self.set_expand(True)
             self.set_resizable(True)
             self.set_fixed_width(1)
@@ -266,13 +311,11 @@ class SongList(AllTreeView, util.InstanceTracker):
         # Render ~#rating directly (simplifies filtering, saves
         # a function call).
         def _cdf(self, column, cell, model, iter, tag):
-            try:
-                song = model.get_value(iter, 0)
-                cell.set_property(
-                    'text', util.format_rating(song.get("~#rating", 0.5)))
+                value = model[iter][0].get("~#rating", const.DEFAULT_RATING)
+                if not self._needs_update(value): return
+                cell.set_property('text', util.format_rating(value))
                 # No need to update layout, we know this width at
                 # at startup.
-            except AttributeError: pass
 
         def __init__(self):
             super(SongList.RatingColumn, self).__init__("~#rating")
@@ -284,52 +327,52 @@ class SongList(AllTreeView, util.InstanceTracker):
         # Optimize for non-synthesized keys by grabbing them directly.
         # Used for any tag without a '~' except 'title'.
         def _cdf(self, column, cell, model, iter, tag):
-            try:
-                song = model.get_value(iter, 0)
-                cell.set_property(
-                    'text', song.get(tag, "").replace("\n", ", "))
-            except AttributeError: pass
+            value = model[iter][0].get(tag, "")
+            if not self._needs_update(value): return
+            cell.set_property('text', value.replace("\n", ", "))
 
     class FSColumn(WideTextColumn):
         # Contains text in the filesystem encoding, so needs to be
         # decoded safely (and also more slowly).
         def _cdf(self, column, cell, model, iter, tag, code=const.FSCODING):
-            try:
-                song = model.get_value(iter, 0)
-                cell.set_property('text', util.unexpand(
-                    song.comma(tag).decode(code, 'replace')))
-            except AttributeError: pass
+            value = model[iter][0].comma(tag)
+            if not self._needs_update(value): return
+            cell.set_property('text', util.unexpand(
+                value.decode(code, 'replace')))
 
     class NumericColumn(TextColumn):
         # Any '~#' keys except dates.
-        _render = gtk.CellRendererText()
-        _render.set_property('xalign', 1.0)
+        def _cdf(self, column, cell, model, iter, tag):
+            value = model[iter][0].comma(tag)
+            if not self._needs_update(value): return
+            text = unicode(value)
+            cell.set_property('text', text)
+            self._update_layout(text, cell)
 
         def __init__(self, tag):
             super(SongList.NumericColumn, self).__init__(tag)
+            self._render.set_property('xalign', 1.0)
             self.set_alignment(1.0)
 
     class LengthColumn(NumericColumn):
         def _cdf(self, column, cell, model, iter, tag):
-            try:
-                song = model.get_value(iter, 0)
-                text = util.format_time(song.get("~#length", 0))
-                cell.set_property('text', text)
-                self._update_layout(text, cell)
-            except AttributeError: pass
-
+            value = model[iter][0].get("~#length", 0)
+            if not self._needs_update(value): return
+            text = util.format_time(value)
+            cell.set_property('text', text)
+            self._update_layout(text, cell)
         def __init__(self):
             super(SongList.LengthColumn, self).__init__("~#length")
 
     class PatternColumn(WideTextColumn):
         def _cdf(self, column, cell, model, iter, tag):
-            try:
-                song = model.get_value(iter, 0)
-                cell.set_property('text', self._pattern % song)
-            except AttributeError: pass
+            song = model.get_value(iter, 0)
+            value = self._pattern % song
+            if not self._needs_update(value): return
+            cell.set_property('text', value)
 
         def __init__(self, pattern):
-            super(SongList.PatternColumn, self).__init__(_("pattern"))
+            super(SongList.PatternColumn, self).__init__(util.pattern(pattern))
             self.header_name = pattern
             self._pattern = Pattern(pattern)
 
@@ -383,8 +426,14 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.set_fixed_height_mode(True)
         self.set_column_headers(self.headers)
         librarian = library.librarian or library
-        sigs = [librarian.connect('changed', self.__song_updated),
-                librarian.connect('removed', self.__song_removed)]
+        sigs = []
+        # The player needs to be called first so it can ge the next song
+        # in case the current one gets deleted and the order gets reset.
+        if player:
+            s = librarian.connect_object('removed', map, player.remove)
+            sigs.append(s)
+        sigs.extend([librarian.connect('changed', self.__song_updated),
+                librarian.connect('removed', self.__song_removed)])
         for sig in sigs:
             self.connect_object('destroy', librarian.disconnect, sig)
         if player:
@@ -398,8 +447,7 @@ class SongList(AllTreeView, util.InstanceTracker):
 
         self.disable_drop()
         self.connect('drag-motion', self.__drag_motion)
-        self.connect(
-            'drag-leave', lambda s, ctx, time: s.parent.drag_unhighlight())
+        self.connect('drag-leave', self.__drag_leave)
         self.connect('drag-data-get', self.__drag_data_get)
         self.connect('drag-data-received', self.__drag_data_received, library)
 
@@ -414,6 +462,16 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.accelerators.connect_group(
             key, mod, 0, lambda *args: self.__information(librarian))
 
+        self.__scroll_delay = None
+        self.__scroll_periodic = None
+        self.__scroll_args = (0, 0, 0, 0)
+        self.__scroll_length = 0
+        self.__scroll_last = None
+
+        # If the a song changes, we can't simply reverse the model
+        # (same tag, changed song order)
+        self.__sort_dirty = False
+
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
             value = model.get_value(iter, 0)(column.header_name)
@@ -438,13 +496,115 @@ class SongList(AllTreeView, util.InstanceTracker):
             gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
         self.drag_dest_unset()
 
+    def __enable_scroll(self):
+        """Start scrolling if it hasn't already"""
+        if self.__scroll_periodic is not None or \
+            self.__scroll_delay is not None:
+            return
+
+        def periodic_scroll():
+            """Get the tree coords for 0,0 and scroll from there"""
+            wx, wy, dist, ref = self.__scroll_args
+            x, y = self.widget_to_tree_coords(0, 0)
+
+            # We reached an end, stop
+            if self.__scroll_last == y:
+                self.__disable_scroll()
+                return
+            self.__scroll_last = y
+
+            # If we went full speed for a while.. speed up
+            # .. every number is made up here
+            if self.__scroll_length >= 50 * ref:
+                dist *= self.__scroll_length / (ref * 10)
+            if self.__scroll_length < 2000 * ref:
+                self.__scroll_length += abs(dist)
+
+            self.scroll_to_point(-1, y + dist)
+            self.__drag_set_dest(wx, wy)
+            # we have to readd the timeout.. otherwise they could add up
+            # because scroll can last longer than 50ms
+            gobject.source_remove(self.__scroll_periodic)
+            enable_periodic_scroll()
+
+        def enable_periodic_scroll():
+            self.__scroll_periodic = gobject.timeout_add(50, periodic_scroll)
+
+        self.__scroll_delay = gobject.timeout_add(350, enable_periodic_scroll)
+
+    def __disable_scroll(self):
+        if self.__scroll_periodic is not None:
+            gobject.source_remove(self.__scroll_periodic)
+            self.__scroll_periodic = None
+        if self.__scroll_delay is not None:
+            gobject.source_remove(self.__scroll_delay)
+            self.__scroll_delay = None
+        self.__scroll_length = 0
+        self.__scroll_last = None
+
+    def __scroll_motion(self, x, y):
+        # TODO: move it views so every treeview can use it
+        visible_rect = self.get_visible_rect()
+        if visible_rect is None:
+            self.__disable_scroll()
+            return
+
+        # I guess the bin to visible_rect difference is the header height
+        # but this could be wrong
+        start = self.get_bin_window().get_geometry()[1] - 1
+        end = visible_rect.height + start
+
+        # Get the font height as size reference
+        reference = self.create_pango_layout("").get_pixel_size()[1]
+
+        # If the drag is in the scroll area, adjust the speed
+        scroll_offset = int(reference * 3)
+        in_upper_scroll = (start < y < start + scroll_offset)
+        in_lower_scroll = (y > end - scroll_offset)
+
+        # thanks TI200
+        accel = lambda x: int(1.1**(x*12/reference)) - (x/reference)
+        if in_lower_scroll:
+            diff = accel(y - end + scroll_offset)
+        elif in_upper_scroll:
+            diff = - accel(start + scroll_offset - y)
+        else:
+            self.__disable_scroll()
+            return
+
+        # The area where we can go to full speed
+        full_offset = int(reference * 0.8)
+        in_upper_full = (start < y < start + full_offset)
+        in_lower_full = (y > end - full_offset)
+        if not in_upper_full and not in_lower_full:
+            self.__scroll_length = 0
+
+        # For the periodic scroll function
+        self.__scroll_args = (x, y, diff, reference)
+
+        # The area to trigger a scroll is a bit smaller
+        trigger_offset = int(reference * 2.5)
+        in_upper_trigger = (start < y < start + trigger_offset)
+        in_lower_trigger = (y > end - trigger_offset)
+
+        if in_upper_trigger or in_lower_trigger:
+            self.__enable_scroll()
+
+    def __drag_leave(self, widget, ctx, time):
+        widget.parent.drag_unhighlight()
+        self.__disable_scroll()
+
+    def __drag_set_dest(self, x, y):
+        try: self.set_drag_dest_row(*self.get_dest_row_at_pos(x, y))
+        except TypeError:
+            if len(self.get_model()) == 0: path = 0
+            else: path = len(self.get_model()) - 1
+            self.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_AFTER)
+
     def __drag_motion(self, view, ctx, x, y, time):
         if self.__drop_by_row:
-            try: self.set_drag_dest_row(*self.get_dest_row_at_pos(x, y))
-            except TypeError:
-                if len(self.get_model()) == 0: path = 0
-                else: path = len(self.get_model()) - 1
-                self.set_drag_dest_row(path, gtk.TREE_VIEW_DROP_AFTER)
+            self.__drag_set_dest(x, y)
+            self.__scroll_motion(x, y)
             if ctx.get_source_widget() == self: kind = gtk.gdk.ACTION_MOVE
             else: kind = gtk.gdk.ACTION_COPY
             ctx.drag_status(kind, time)
@@ -577,7 +737,7 @@ class SongList(AllTreeView, util.InstanceTracker):
             count = int(float(cellx - 5) / width) + 1
             rating = max(0.0, min(1.0, count * util.RATING_PRECISION))
             if (rating <= util.RATING_PRECISION and
-                song["~#rating"] == util.RATING_PRECISION): rating = 0
+                song("~#rating") == util.RATING_PRECISION): rating = 0
             self.__set_rating(rating, [song], librarian)
 
     def __set_rating(self, value, songs, librarian):
@@ -636,19 +796,19 @@ class SongList(AllTreeView, util.InstanceTracker):
     def get_sort_by(self):
         for header in self.get_columns():
             if header.get_sort_indicator():
-                try: tag = header._pattern.format
-                except AttributeError: tag = header.header_name
+                tag = header.header_name
                 sort = header.get_sort_order()
                 return (tag, sort == gtk.SORT_DESCENDING)
         else: return "album", False
 
     def is_sorted(self):
-        return max([col.get_sort_indicator() for col in self.get_columns()])
+        return max([c.get_sort_indicator() for c in self.get_columns()] or [0])
 
     # Resort based on the header clicked.
     def set_sort_by(self, header, tag=None, order=None, refresh=True):
         if header and tag is None: tag = header.header_name
 
+        rev = False
         for h in self.get_columns():
             if h.header_name == tag:
                 if order is None:
@@ -660,10 +820,20 @@ class SongList(AllTreeView, util.InstanceTracker):
                 else:
                     if order: s = gtk.SORT_DESCENDING
                     else: s = gtk.SORT_ASCENDING
+                rev = h.get_sort_indicator()
                 h.set_sort_indicator(True)
                 h.set_sort_order(s)
             else: h.set_sort_indicator(False)
-        if refresh: self.set_songs(self.get_songs())
+        if refresh:
+            if rev:
+                if self.__sort_dirty:
+                    # python sort is faster if it's presorted.
+                    songs = self.get_songs()
+                    songs.reverse()
+                    self.set_songs(songs)
+                else:
+                    self.reverse()
+            else: self.set_songs(self.get_songs())
 
     def set_model(self, model):
         super(SongList, self).set_model(model)
@@ -674,23 +844,53 @@ class SongList(AllTreeView, util.InstanceTracker):
         try: return self.get_model().get()
         except AttributeError: return [] # model is None
 
+    def __get_sort_tag(self, tag):
+        replace_order = {
+            "~#track": "album",
+            "~#disc": "album",
+            "~length": "~#length"
+        }
+
+        if tag == "~title~version":
+            tag = "title"
+        elif tag == "~album~discsubtitle":
+            tag = "album"
+
+        if tag.startswith("<"):
+            for key, value in replace_order.iteritems():
+                tag = tag.replace("<%s>" % key, "<%s>" % value)
+            tag = Pattern(tag).format
+        else:
+            tags = util.tagsplit(tag)
+            sort_tags = []
+            for tag in tags:
+                tag = replace_order.get(tag, tag)
+                tag = TAG_TO_SORT.get(tag, tag)
+                if tag not in sort_tags:
+                    sort_tags.append(tag)
+            if len(sort_tags) > 1:
+                tag = "~" + "~".join(sort_tags)
+
+        return tag
+
     def set_songs(self, songs, sorted=False):
         model = self.get_model()
+        self.__sort_dirty = False
 
         if not sorted:
             tag, reverse = self.get_sort_by()
-            if tag == "~#track": tag = "album"
-            elif tag == "~#disc": tag = "album"
-            elif tag == "~length": tag = "~#length"
-            elif tag == "~album~discsubtitle": tag = "album"
+            tag = self.__get_sort_tag(tag)
 
-            tag = TAG_TO_SORT.get(tag, tag)
+            #try to set a sort indicator that matches the default order
+            if not self.is_sorted():
+                for h in self.get_columns():
+                    name = h.header_name
+                    if self.__get_sort_tag(name) == tag:
+                        h.set_sort_indicator(True)
+                        h.set_sort_order(gtk.SORT_ASCENDING)
+                        break
 
-            if callable(tag):
-                # A pattern is currently selected.
-                sort_func = lambda song: (util.tag(song), song.sort_key, song)
-            else:
-                sort_func = lambda song: (song(tag), song.sort_key, song)
+            sort_func = AudioFile.sort_by_func(tag)
             songs.sort(key=sort_func, reverse=reverse)
         else:
             self.set_sort_by(None, refresh=False)
@@ -706,13 +906,33 @@ class SongList(AllTreeView, util.InstanceTracker):
         print_d("Model attached.", context=self)
         map(gtk.TreeViewColumn.set_sort_indicator, self.get_columns(), sorts)
 
+    def reverse(self):
+        model = self.get_model()
+        sorts = map(gtk.TreeViewColumn.get_sort_indicator, self.get_columns())
+        self.set_model(None)
+        model.reverse()
+        self.set_model(model)
+        map(gtk.TreeViewColumn.set_sort_indicator, self.get_columns(), sorts)
+
     def get_selected_songs(self):
-        model, rows = self.get_selection().get_selected_rows()
+        selection = self.get_selection()
+        if selection is None: return []
+        model, rows = selection.get_selected_rows()
         return [model[row][0] for row in rows]
 
     def __song_updated(self, librarian, songs):
+        """Only update rows that are currently displayed.
+        Warning: This makes the row-changed signal useless."""
+        self.__sort_dirty = True
+        #pygtk 2.12: prevent invalid ranges or GTK asserts
+        if not self.flags() & gtk.REALIZED or \
+            self.get_path_at_pos(0,0) is None: return
+        vrange = self.get_visible_range()
+        if vrange is None: return
+        (start,), (end,) = vrange
         model = self.get_model()
-        for row in model:
+        for path in xrange(start, end+1):
+            row = model[path]
             if row[0] in songs:
                 model.row_changed(row.path, row.iter)
 
@@ -747,6 +967,8 @@ class SongList(AllTreeView, util.InstanceTracker):
     # new values.
     def set_column_headers(self, headers):
         if len(headers) == 0: return
+
+        old_sort = self.is_sorted() and self.get_sort_by()
         map(self.remove_column, self.get_columns())
 
         if self.CurrentColumn is not None:
@@ -773,13 +995,21 @@ class SongList(AllTreeView, util.InstanceTracker):
             column.set_reorderable(True)
             self.append_column(column)
 
+        if old_sort:
+            header, order = old_sort
+            self.set_sort_by(None, header, order, False)
+
     def __getmenu(self, column):
         menu = gtk.Menu()
         menu.connect_object('selection-done', gtk.Menu.destroy, menu)
 
         current = SongList.headers[:]
         current_set = set(current)
-        current = zip(map(util.tag, current), current)
+        def tag_title(tag):
+            if tag.startswith("<"):
+                return util.pattern(tag)
+            return util.tag(tag)
+        current = zip(map(tag_title, current), current)
 
         def add_header_toggle(menu, (header, tag), active, column=column):
             item = gtk.CheckMenuItem(header)
@@ -854,7 +1084,7 @@ class SongList(AllTreeView, util.InstanceTracker):
         from quodlibet.qltk.prefs import PreferencesWindow
         win = PreferencesWindow(self)
         win.child.set_current_page(0)
-        
+
     def __showmenu(self, column, event=None):
         time = gtk.get_current_event_time()
         if event is not None and event.button != 3:
