@@ -17,6 +17,9 @@ from quodlibet.util.titlecase import title
 
 from quodlibet.const import FSCODING as fscoding, ENCODING
 
+if os.name == "nt":
+    from win32com.shell import shellcon, shell
+
 def strip_win32_incompat(string, BAD = '\:*?;"<>|'):
     """Strip Win32-incompatible characters.
 
@@ -98,7 +101,7 @@ class OptionParser(object):
             if self.__args[opt]:
                 opt = "%s=%s" % (opt, self.__args[opt])
             return "  --%s %s\n" % (opt.ljust(space), help)
-                
+
         else: return ""
 
     def help(self):
@@ -230,7 +233,12 @@ def format_rating(value):
 
 def format_size(size):
     """Turn an integer size value into something human-readable."""
-    if size >= 1024*1024 * 10:
+    # TODO: Better i18n of this (eg use O/KO/MO/GO in French)
+    if size >= 1024*1024*1024:
+        return "%.1fGB" % (float(size) / (1024*1024*1024))
+    elif size >= 1024*1024 * 100:
+        return "%.0fMB" % (float(size) / (1024*1024))
+    elif size >= 1024*1024 * 10:
         return "%.1fMB" % (float(size) / (1024*1024))
     elif size >= 1024*1024:
         return "%.2fMB" % (float(size) / (1024*1024))
@@ -286,6 +294,20 @@ def fsencode(s):
     if isinstance(s, str): return s
     else: return s.encode(fscoding, 'replace')
 
+"""
+Path related functions like open, os.listdir have different behavior on win32
+
+- Passing a string calls the old non unicode win API.
+  In case of listdir this leads to "?" for >1byte chars and to
+  1 byte chars encoded using the fs encoding. -> DO NOT USE!
+
+- Passing a unicode object internally calls the windows unicode functions.
+  This will mostly lead to proper unicode paths (except expanduser).
+
+  And that's why QL is using unicode paths on win and encoded paths
+  everywhere else.
+"""
+
 if sys.platform == "win32":
     fsnative = fsdecode # Decode a filename on windows
 else:
@@ -392,8 +414,8 @@ def split_album(s):
 
 def split_numeric(s, reg=re.compile(r"([0-9]+\.?[0-9]*)")):
     """Seperate numeric values from the string and convert to float, so
-    it can be used for human sorting. Also removes all whitespace."""
-    if not s: return None
+    it can be used for human sorting. Also removes all extra whitespace."""
+    if not s: return ('',)
     result = reg.search(s)
     if not result: return (u" ".join(s.split()),)
     else:
@@ -405,7 +427,7 @@ def human_sort_key(s):
     if not isinstance(s, unicode):
         s = s.decode("utf-8")
     s = unicodedata.normalize('NFD', s.lower())
-    return split_numeric(s)
+    return split_numeric(s[:1024])
 
 def find_subtitle(title):
     if isinstance(title, str): title = title.decode('utf-8', 'replace')
@@ -419,17 +441,28 @@ def find_subtitle(title):
                 return title.rstrip(), subtitle
     else: return title, None
 
-def unexpand(filename, HOME=os.path.expanduser("~")):
+def expanduser(filename):
+    """needed because expanduser does not return wide character paths
+    on windows even if a unicode path gets passed."""
+    if os.name == "nt":
+        profile = shell.SHGetFolderPath(0, shellcon.CSIDL_PROFILE, 0, 0)
+        if filename == "~": return profile
+        if filename.startswith(u"~" + os.path.sep):
+            return os.path.join(profile, filename[2:])
+    return os.path.expanduser(filename)
+
+def unexpand(filename, HOME=expanduser("~")):
     """Replace the user's home directory with ~/, if it appears at the
     start of the path name."""
-    if filename == HOME: return "~"
-    elif filename.startswith(HOME + "/"):
-        filename = filename.replace(HOME, "~", 1)
+    sub = (os.name == "nt" and "%USERPROFILE%") or "~"
+    if filename == HOME: return sub
+    elif filename.startswith(HOME + os.path.sep):
+        filename = filename.replace(HOME, sub, 1)
     return filename
 
 def website(site):
     site = site.replace("\\", "\\\\").replace("\"", "\\\"")
-    for prog in (["xdg-open", "gnome-open", "sensible-browser"] +
+    for prog in (["gnome-open", "xdg-open", "sensible-browser"] +
               os.environ.get("BROWSER","").split(":")):
         if iscommand(prog):
             args = prog.split()
@@ -477,20 +510,22 @@ def tagsplit(tag):
         return tags
     else: return [tag]
 
-def pattern(pat, cap=True):
+def pattern(pat, cap=True, esc=False):
     """Return a 'natural' version of the pattern string for human-readable
     bits. Assumes all tags in the pattern are present."""
-    from quodlibet.parse import Pattern
+    from quodlibet.parse import Pattern, XMLFromPattern
     class Fakesong(dict):
+        cap = False
         def comma(self, key):
-            if key.startswith('~') and '~' in key[1:]:
-                tags = [self.get(k, '') for k in tagsplit(key)]
-                return " - ".join(tags)
-            return self.get(key, '')
+            return " - ".join(self.list(key))
+        def list(self, key):
+            return [tag(k, self.cap) for k in tagsplit(key)]
+        list_seperate = list
+        __call__ = comma
+
     fakesong = Fakesong({'filename': tag('filename', cap)})
-    p = Pattern(pat)
-    for t in p.real_tags():
-        fakesong.setdefault(t, tag(t, cap))
+    fakesong.cap = cap
+    p = (esc and XMLFromPattern(pat)) or Pattern(pat)
     return p.format(fakesong)
 
 def spawn(argv, stdout=False):
@@ -556,10 +591,68 @@ class DeferredSignal(object):
         self.func(*args)
         self.dirty = False
 
+def gobject_weak(fun, *args, **kwargs):
+    """Connect to a signal and disconnect if destroy gets emitted.
+    If parent is given, it connects to its destroy signal
+    Example:
+        gobject_weak(gobject_1.connect, 'changed', self.__changed)
+        gobject_weak(gobject_1.connect, 'changed', self.__changed,
+            parent=gobject_2)
+    """
+    parent = kwargs.pop("parent", None)
+    obj = fun.__self__
+    sig = fun(*args)
+    disconnect = lambda obj, handle: obj.disconnect(handle)
+    if parent: parent.connect_object('destroy', disconnect, obj, sig)
+    else: obj.connect('destroy', disconnect, sig)
+    return sig
+
+class cached_property(object):
+    """A read-only @property that is only evaluated once."""
+    def __init__(self, fget, doc=None):
+        self.fget = fget
+        self.__doc__ = doc or fget.__doc__
+        self.__name__ = fget.__name__
+
+    def __get__(self, obj, cls):
+        if obj is None:
+            return self
+        obj.__dict__[self.__name__] = result = self.fget(obj)
+        return result
+
 def xdg_get_system_data_dirs():
     """http://standards.freedesktop.org/basedir-spec/latest/"""
     data_dirs = os.getenv("XDG_DATA_DIRS")
     if data_dirs:
-        return data_dirs.split(":")
+        return map(os.path.abspath, data_dirs.split(":"))
     else:
         return ("/usr/local/share/", "/usr/share/")
+
+def xdg_get_data_home():
+    data_home = os.getenv("XDG_DATA_HOME")
+    if data_home:
+        return os.path.abspath(data_home)
+    else:
+        return os.path.join(os.path.expanduser("~"), ".local", "share")
+
+def find_mount_point(path):
+    while not os.path.ismount(path):
+        path = os.path.dirname(path)
+    return path
+
+def pathname2url_win32(path):
+    # stdlib version raises IOError for more than one ':' which can appear
+    # using a virtual box shared folder and it inserts /// at the beginning
+    # but it should be /.
+    quote = urllib.quote
+    if path[1:2] != ":" or path[:1] == "\\":
+        if path[:2] == "\\\\":
+            path = path[2:]
+        return quote("/".join(path.split("\\")))
+    drive, remain = path.split(":", 1)
+    return "/%s:%s" % (quote(drive), quote("/".join(remain.split("\\"))))
+
+if os.name == "nt":
+    pathname2url = pathname2url_win32
+else:
+    pathname2url = urllib.pathname2url
