@@ -127,7 +127,7 @@ add(Tdecode)
 class Tencode(TestCase):
     def test_empty(self):
         self.failUnlessEqual(util.encode(""), "")
-    def test_unicode(self):        
+    def test_unicode(self):
         self.failUnlessEqual(util.encode(u"abcde"), "abcde")
 add(Tencode)
 
@@ -247,7 +247,12 @@ class Tformat_size(TestCase):
 
     def test_mbytes(self):
         self.t_dict({ 1024*1024: "1.00MB", 1024*1536: "1.50MB",
-                      1024*10240: "10.0MB", 1024*15360: "15.0MB"})
+                      1024*10240: "10.0MB", 1024*15360: "15.0MB",
+                      123456*1024: "121MB", 765432*1024: "747MB"})
+
+    def test_gbytes(self):
+        self.t_dict({ 1024*1024*1024: "1.0GB", 1024*1024*1536: "1.5GB",
+                      1024*1024*10240: "10.0GB", 1024*1024*15360: "15.0GB"})
 add(Tformat_size)
 
 class Tsplit_title(TestCase):
@@ -274,7 +279,7 @@ class Tsplit_album(TestCase):
         self.failUnlessEqual(
             util.split_album("disk 2"), ("disk 2", None))
 
-    def test_basic_disc(self):        
+    def test_basic_disc(self):
         self.failUnlessEqual(
             util.split_album("foo disc 1/2"), ("foo", "1/2"))
 
@@ -373,8 +378,11 @@ class Tpattern(TestCase):
     def test_condition(self):
         self.failUnlessEqual(util.pattern("<~year|<~year> - <album>|<album>>"),
                              "Year - Album")
+    def test_escape(self):
+        self.failUnlessEqual(util.pattern("\<i\><&>\</i\>", esc=True),
+                            "<i>&amp;</i>")
     def test_invalid(self):
-        self.failUnlessEqual(util.pattern("<date"), "<date")
+        self.failUnlessEqual(util.pattern("<date"), "")
 add(Tpattern)
 
 class Tformat_time_long(TestCase):
@@ -427,3 +435,33 @@ class Tspawn(TestCase):
         fileobj = util.spawn(["echo", "'$1'", '"$2"', ">3"], stdout=True)
         self.failUnlessEqual(fileobj.read().split(), ["'$1'", '"$2"', ">3"])
 add(Tspawn)
+
+class Txdg_dirs(TestCase):
+    def test_system_data_dirs(self):
+        os.environ["XDG_DATA_DIRS"] = "/xyz"
+        self.failUnlessEqual(util.xdg_get_system_data_dirs()[0], "/xyz")
+        del os.environ["XDG_DATA_DIRS"]
+        dirs = util.xdg_get_system_data_dirs()
+        self.failUnlessEqual(dirs[0], "/usr/local/share/")
+        self.failUnlessEqual(dirs[1], "/usr/share/")
+
+    def test_data_home(self):
+        os.environ["XDG_DATA_HOME"] = "/xyz"
+        self.failUnlessEqual(util.xdg_get_data_home(), "/xyz")
+        del os.environ["XDG_DATA_HOME"]
+        should = os.path.join(os.path.expanduser("~"), ".local", "share")
+        self.failUnlessEqual(util.xdg_get_data_home(), should)
+add(Txdg_dirs)
+
+class Tpathname2url(TestCase):
+    def test_win(self):
+        cases = {
+            r"c:\abc\def" : "/c:/abc/def",
+            r"C:\a b\c.txt": "/C:/a%20b/c.txt",
+            r"\\xy\z.txt": "xy/z.txt",
+            r"C:\a:b\c:d": "/C:/a%3Ab/c%3Ad"
+            }
+        p2u = util.pathname2url_win32
+        for inp, should in cases.iteritems():
+            self.failUnlessEqual(p2u(inp), should)
+add(Tpathname2url)
